/*
 * File:       cutTex.C
 * Author:     $Author$
 * Created:    Fri Jun  6 1997
 * Revision:   $Revision$
 * Updated:    $Date$
 * Copyright Remigiusz Trzaska 1997 NPAC, Syracuse University
 *
 */

#include <fstream.h>
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

int degSize;
int threshold[20];
float north;
float west;
int dimX, dimY, colorRes;
int degWest, minWest, degNorth, minNorth, degLon, minLon, degLat, minLat;
char** dataR;
char** dataG;
char** dataB;

int
parseFileName(char* fileName)
{
   sscanf(fileName, "%d:%d,%d:%d,%d:%d,%d:%d.ppm",
	  &degWest, &minWest, &degNorth, &minNorth,
	  &degLon, &minLon, &degLat, &minLat);
   return 1;
}

int
readImage(char* fileName)
{
   char charJunk;
   
   ifstream image(fileName);

   // P5\n
   image.get(charJunk);
   image.get(charJunk);
   image.get(charJunk);

   image >> dimX >> dimY >> colorRes;

   cerr << dimX << dimY << colorRes;

   if(colorRes != 255)
   {
      cerr << "A problem reading PPM header!\n";
      return 0;
   }

   dataR = new char* [dimY];
   dataG = new char* [dimY];
   dataB = new char* [dimY];
   for(int i=0; i<dimY; i++)
   {
      dataR[i] = new char[dimX];
      dataG[i] = new char[dimX];
      dataB[i] = new char[dimX];
   }

   image.get(charJunk);

   for(int row=0; row<dimY; row++)
      for(int col=0; col<dimX; col++)
      {
	 image.get(charJunk);
	 dataR[row][col] = charJunk;
	 image.get(charJunk);
	 dataG[row][col] = charJunk;
	 image.get(charJunk);
	 dataB[row][col] = charJunk;
      }
   
   image.close();

   cerr << "Read image\n";
   
   return 1;
}

char*
GetNameForLocation(int row, int col)
{
   int minutesLon = degWest * 60 + minWest - degSize * col;
   int minutesLat = degNorth * 60 + minNorth - degSize * row;
   char buf[256];
   char buf1[256];
   cerr << "For row " << row << " and col " << col << ' ';
   
   sprintf(buf, "%d:%d:0,%d:%d:0.ppm", minutesLon/60, minutesLon%60,
	   minutesLat/60, minutesLat%60);
   int i=0, found=0;
   do
   {
      sprintf(buf1, "_%s%c", buf, (i==0)? '\0': i+'0');
      ifstream testExistance(buf1);
      found = testExistance.good();
      if(!found)
	 cerr << buf1 << " not found\n";
      i++;
   } while(found);

   return buf1;
};

int
cutImage()
{
   int xCount = (degLon*60 + minLon)/degSize;
   int yCount = (degLat*60 + minLat)/degSize;

   for(int row=0; row<yCount; row++)
      for(int col=0; col<xCount; col++)
      {
	 char* name = GetNameForLocation(row, col);
	 cerr << "Name " << name << endl;
	 cerr << "Row offset: " << threshold[row];
	 cerr << " Col offset: " << dimX/xCount*col;
	 
	 cerr << endl << endl;
	 
	 ofstream chunk(name);
	 chunk << "P6\n"
	       << dimX/xCount << " " << dimY/yCount << "\n255\n";
	 for(int i=0; i<dimY/yCount; i++)
	    for(int j=0; j<dimX/xCount; j++)
	    {
	       cerr << "[" << threshold[row]+i << "][" << dimX/xCount*col+j << "] ";
	       
	       chunk.put(dataR[threshold[row]+i][dimX/xCount*col+j]);
	       chunk.put(dataG[threshold[row]+i][dimX/xCount*col+j]);
	       chunk.put(dataB[threshold[row]+i][dimX/xCount*col+j]);
	    }
	 chunk.close();
      }
   return 1;
}

void
main(int argc, char* argv[])
{
   if(argv[1][0]=='-' && argv[1][1]=='h')
   {
      cerr << "Usage:\n" << argv[0] << " ppmfile degSize thresholds\n\n";
      return;
   };

   parseFileName(argv[1]);
   cerr << "Parsed file name\n";
   
   degSize = atoi(argv[2]);
   cerr << "Parsed degSize\n";

   readImage(argv[1]);
   cerr << "Read image\n";

   threshold[0] = 0;
   if(argc==3)
   {
      for(int i=1; i<(degLat*60+minLat)/degSize; i++)
      {
	 threshold[i] = (i+1)*degSize*dimY/(degLon*60+minLon);
	 cerr << "threshold[" << i << "]=" << threshold[i] << endl;
      }
      threshold[i] = dimY;
      cerr << "threshold[" << i << "]=" << threshold[i] << endl;
   }
   else
   {
      for(int argument = 3; argument<argc; argument++)
      {
	 threshold[argument-2] = atoi(argv[argument]);
	 cerr << "threshold[" << argument-2 << "]=" << threshold[argument-2] << endl;
      }
      threshold[argument-2] = dimY;
   }
   

   cutImage();
   
};
